class Billable
  def charge(rate, timestamp)
    hours, minutes = split_timestamp(timestamp)
    decimal = minutes / 60.0

    rate * (hours + decimal)
  end

  def sum(args={})
    period_x = args.fetch(:period_x, '0:00') || '0:00'
    period_y = args.fetch(:period_y, '0:00') || '0:00'

    hour_x, min_x = split_timestamp(period_x)
    hour_y, min_y = split_timestamp(period_y)

    hours_sum = hour_x + hour_y
    min_sum = min_x + min_y

    hours, minutes = accumulate_hours_from_minutes(hours_sum, min_sum)

    format_timestamp(hours, minutes)
  end

  def sum_multiple(args)
    timestamps = args.fetch(:timestamps, [])
    reduce_timestamps(timestamps)
  end

  def accumulate_hours_from_minutes(hours, minutes)
    if minutes >= 60
      return accumulate_hours_from_minutes(hours + 1, minutes - 60)
    end

    [hours, minutes]
  end

  private

  def reduce_timestamps(timestamps)
    return sum(period_x: timestamps.first) if timestamps.length == 1
    return reduce_timestamps(map_sum_to(timestamps))
  end

  def map_sum_to(timestamps)
    timestamps.each_slice(2).map { |x, y| sum(period_x: x, period_y: y) }
  end

  def split_timestamp(timestamp)
    timestamp.split(':').map(&:to_i)
  end

  def format_timestamp(hours, minutes)
    minutes = minutes.to_s.rjust(2, '0')
    "#{hours}:#{minutes}"
  end
end
